LANG_ALIASES = {
    "english": "eng_Latn",
    "eng": "eng_Latn",
    "en": "eng_Latn",
    "ko": "kor_Hang",
    "korean": "kor_Hang",
    "kor": "kor_Hang",
    "kr": "kor_Hang"
}
LEVEL = {
    "fine": "fine",
    "fine-tuned": "fine",
    "origin": "origin",
    "0": "origin",
    "1": "fine"
}
STYLE = {
    "formal":"문어체",
    "informal":"구어체",
    "android":"안드로이드",
    "azae":"아재",
    "chat":"채팅",
    "choding":"초등학생",
    "emoticon":"이모티콘",
    "enfp":"enfp",
    "gentle":"신사",
    "halbae":"할아버지",
    "halmae":"할머니",
    "joongding":"중학생",
    "king":"왕",
    "naruto":"나루토",
    "seonbi":"선비",
    "sosim":"소심한",
    "translator":"번역기",
}